package org.gridminer.jspace.config;


import org.gridminer.jspace.manager.PersistentOntology;
import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.ModelMaker;

/**
 * 
 * This class is for setting up the RDF store and the persistent ontology model 
 * 
 * */
public class JSpaceSetup {
   
	
    //the ontology URI
   	//e-Science life cycle ontology with a complete single life cycle resource (LCR1) as individuals and properties
    public static final String ONT1 = "http://www.gridminer.org/e-sciencelifecycle/lifecycleontology_v3.0_LCR1.owl";
   
    //MYSQL DATABASE SETTINGS ON VIRTUAL MASCHINE (LOCAL RDF STORE)
    public static final String DB_URL = "jdbc:mysql://localhost:3306/lifecycle";
    public static final String DB_USER = "root";
    public static final String DB_PASSWD = "password";
    public static final String DB = "MySQL";
    public static final String DB_DRIVER = "com.mysql.jdbc.Driver";
    
    // database connection parameters, with defaults
    private static String s_dbURL = DB_URL;
    private static String s_dbUser = DB_USER;
    private static String s_dbPw = DB_PASSWD;
    private static String s_dbType = DB;
    private static String s_dbDriver = DB_DRIVER;

    // if true, reload the data
    private static boolean s_reload = true;

    // source URL to load data from; if null, use default
    private static String s_source = ONT1;

    public static void main(String[] args) {
        //if JSpaceSetup.java is called from the console arguments can be given to configure another RDF Store
    	processArgs(args);
        s_source = ONT1;
        
        // create the helper class we use to handle the persistent ontologies
        PersistentOntology po = new PersistentOntology();

        // ensure the JDBC driver class is loaded
        try {
            Class.forName( s_dbDriver );
        }
        catch (Exception e) {
            System.err.println( "Failed to load the driver for the database: " + e.getMessage() );
            System.err.println( "Have you got the CLASSPATH set correctly?" );
        }

        // are we re-loading the data this time?
        if (s_reload) {
        	// we pass cleanDB=true to clear out existing models
            // NOTE: this will remove ALL Jena models from the named persistent store, so
            // use with care if you have existing data stored
            ModelMaker maker = po.getRDBMaker( s_dbURL, s_dbUser, s_dbPw, s_dbType, true );
            System.out.println("Cleaning local RDF Store ...");

            long start0 = System.currentTimeMillis();
            // now load the source data into the newly cleaned db
            po.loadDB( maker, s_source );
            System.out.println("Load time: " + (System.currentTimeMillis() - start0) + " msec.");
        }

        ModelMaker maker = po.getRDBMaker( s_dbURL, s_dbUser, s_dbPw, s_dbType, false );
        
        long start2 = System.currentTimeMillis();
        po.executeSparqlQuery( maker, s_source );
        System.out.println("Responce time (Query Execution): " + (System.currentTimeMillis() - start2) + " msec.");
        
    }


    /**
     * Process any command line arguments
     */
    private static void processArgs(String[] args) {
        int i = 0;
        while (i < args.length) {
            String arg = args[i++];

            if      (arg.equals( "--dbUser" ))   {s_dbUser = args[i++];}
            else if (arg.equals( "--dbURL" ))    {s_dbURL = args[i++];}
            else if (arg.equals( "--dbPasswd" )) {s_dbPw = args[i++];}
            else if (arg.equals( "--dbType" ))   {s_dbType = args[i++];}
            else if (arg.equals( "--reload" ))   {s_reload = true;}
            else if (arg.equals( "--dbDriver" )) {s_dbDriver = args[i++];}
            else {
                // assume this is a URL to load data from
                s_source = arg;
            }            
        }
        System.out.println("LOCAL RDF STORE SETTINGS:");
        System.out.println("   dbUser:" + s_dbUser);
        System.out.println("   dbURL:" + s_dbURL);
        System.out.println("   dbPasswd:" + s_dbPw);
        System.out.println("   dbType:" + s_dbType);
        System.out.println("   dbDriver:" + s_dbDriver);
        //System.out.println("s_source:" + s_source);
    }

    /**
     * Answer the default source document, and set up the document manager
     * so that we can find it on the file system
     *
     * @return The URI of the default source document
     */
    private static String getDefaultSource() {
        // use the ont doc mgr to map from a generic URN to a local source file
        OntDocumentManager.getInstance().addAltEntry( ONT1, "file:data/lifecycleontology_v3.0.owl" );
    	OntDocumentManager.getInstance().addAltEntry( ONT1, "file:data/lco1-5.nt" );
    	
    	return ONT1;
    }
  
}


