package org.gridminer.jspace.performance;

import org.gridminer.jspace.lifecycle.LifeCycleResource;
import org.gridminer.jspace.lifecycle.MyUtilities;
import org.gridminer.jspace.manager.PersistentOntology;

import com.hp.hpl.jena.ontology.OntDocumentManager;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.ModelMaker;


public class JSpaceLocalRDFPerformance {
   	public static final String ONT1 = "http://www.gridminer.org/e-sciencelifecycle/lco-1LCR.owl";
  
    //MYSQL DATABASE SETTINGS ON VIRTUAL MASCHINE (LOCAL RDF STORE)
    public static final String DB_URL = "jdbc:mysql://localhost:3306/lifecycle";
    public static final String DB_USER = "root";
    public static final String DB_PASSWD = "password";
    public static final String DB = "MySQL";
    public static final String DB_DRIVER = "com.mysql.jdbc.Driver";

    // database connection parameters, with defaults
    private static String s_dbURL = DB_URL;
    private static String s_dbUser = DB_USER;
    private static String s_dbPw = DB_PASSWD;
    private static String s_dbType = DB;
    private static String s_dbDriver = DB_DRIVER;

    // if true, reload the data
    private static boolean s_reload = false;

    // source URL to load data from; if null, use default
    private static String s_source = ONT1;
    
    public static void main(String[] args) {
     
    	s_source = ONT1;
        
        // create the helper class we use to handle the persistent ontologies
        PersistentOntology po = new PersistentOntology();

        // ensure the JDBC driver class is loaded
        try {
            Class.forName( s_dbDriver );
        }
        catch (Exception e) {
            System.err.println( "Failed to load the driver for the database: " + e.getMessage() );
            System.err.println( "Have you got the CLASSPATH set correctly?" );
        }

        // are we re-loading the data this time?
        if (s_reload) {

            // we pass cleanDB=true to clear out existing models
            // NOTE: this will remove ALL Jena models from the named persistent store, so
            // use with care if you have existing data stored
            ModelMaker maker = po.getRDBMaker( s_dbURL, s_dbUser, s_dbPw, s_dbType, true );
            System.out.println("Cleaning local RDF Store ...");

            long start0 = System.currentTimeMillis();
            // now load the source data into the newly cleaned db
            po.loadDB( maker, s_source );
            System.out.println("Load time: " + (System.currentTimeMillis() - start0) + " msec.");
            
        }

        // some tests: now we list the classes in the database, to show that the persistence worked
        
        ModelMaker maker = po.getRDBMaker( s_dbURL, s_dbUser, s_dbPw, s_dbType, false );
        
        /*
        long start1 = System.currentTimeMillis();
        po.listClasses( maker, s_source );
        System.out.println("Responce time (PO List Classes): " + (System.currentTimeMillis() - start1) + " msec.");
        */
        
        
      
        /*************** Test Queries *************************************/ 
        String query1 = 	"PREFIX owl: <http://localhost/LifeCycleOntology.owl#> SELECT ?lifeCycle " +
			"?goalSpecification ?dataPreparation ?taskSelection WHERE { ?lifeCycle " +
			"owl:hasGoalSpecification ?goalSpecification. ?lifeCycle " +
			"owl:hasTaskSelection ?taskSelection. ?lifeCycle owl:hasDataPreparation " +
			"?dataPreparation. }";
      
		String query2 = "PREFIX owl: <http://localhost/LifeCycleOntology.owl#> SELECT ?s ?p ?o WHERE { ?s ?p ?o }";
		
		String query3 = "PREFIX owl: <http://localhost/LifeCycleOntology.owl#> SELECT ?s ?p WHERE { ?s ?p owl:i.elsayed }";
		
		String query4 = "PREFIX owl: <http://localhost/LifeCycleOntology.owl#> SELECT ?lifeCycle ?goalSpecification ?p ?o WHERE { ?lifeCycle owl:hasGoalSpecification ?goalSpecification. ?lifeCycle ?p ?o. } ";
		
		String query5 = "PREFIX owl: <http://localhost/LifeCycleOntology.owl#> SELECT ?member ?p ?o WHERE { owl:researchGroup_1 owl:hasMembers ?member . ?member ?p ?o }";       
		        
		
        /*************** Execute Queries *************************************/
        long start2 = System.currentTimeMillis();
        ResultSet rs = po.executeTestSparqlQuery( maker, s_source, query1);
        System.out.println("Responce time (Query Execution): " + (System.currentTimeMillis() - start2) + " msec.");
        
       
        /*************** Format Results *************************************/
        long start3 = System.currentTimeMillis();
        System.out.println(po.formatResultSet(rs, "Text", "local"));
        System.out.println("Responce time (Result Format Text): " + (System.currentTimeMillis() - start3) + " msec.");
          
        
        /*************** Create and Insert a Test LCR *************************************/
        long start4 = System.currentTimeMillis();
       
        //create a test LifeCycleResoruce with the metadata from the XML file
		LifeCycleResource lcr = MyUtilities.getLifeCycleFromXMLFile("TestLCR-BGA.xml");
		
		System.out.println(lcr.getLCRName() + " " + lcr.getLCRID());
        
        po.publishLCR(maker, s_source, lcr);
        
		System.out.println("Responce time (created LCR): " + (System.currentTimeMillis() - start4) + " msec.");
        
    }


    /**
     * Answer the default source document, and set up the document manager
     * so that we can find it on the file system
     *
     * @return The URI of the default source document
     */
    private static String getDefaultSource() {
        // use the ont doc mgr to map from a generic URN to a local source file
        //OntDocumentManager.getInstance().addAltEntry( ONT1, "file:data/lifecycleontology_v3.0.owl" );
    	
        /*************** Datasets *************************************/
    	OntDocumentManager.getInstance().addAltEntry( ONT1, "file:data/lco-5000LCRs.owl" );  
    	
    	return ONT1;
    }
    

}


