package org.gridminer.jspace.queryProcessor;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;

import org.dbgrid.aderis.ResultSink;
import org.dbgrid.aderis.planner.Plan;
import org.dbgrid.aderis.planner.PlanListener;

/*
* A result sink implementation for writing to standard output  
*/
public class BufferedWriterResultSink implements ResultSink, PlanListener {
	
private PlanListener listener;
    
    private long startTime;
    private String fileName;
    String header = "";
    
    /**
     * Using this constructor means that no timing info will be displayed.
     */
    public BufferedWriterResultSink() {
        startTime = -1;
    }
    
    /**
     * Using this constructor means that no timing info will be displayed.
     * And that the fileName will be given.
     */
    public BufferedWriterResultSink(String formatType) {
    	startTime = -1;
    	fileName = "global_resultSet_" + formatType;
    	System.out.print("filename in ResultSink: " + fileName);
    	System.out.println();
    	
    	File file = new File(fileName);
		if (file.exists())
			file.delete();
    }
    
    /**
     * Provides the result sink with an initial time. Finish time will be
     * displayed.
     */
    public BufferedWriterResultSink(long startTime) {
        this.startTime = startTime;
    }
    
    /*
     * (non-Javadoc)
     * @see org.dbgrid.aderis.ResultSink#header(java.lang.String[])
     */
    public void header(String[] fields) {
        
    	BufferedWriter bw = null;
    	
    	try {
			boolean append = true;
			bw = new BufferedWriter(new FileWriter(fileName, append));
						
			try {
				
				for(int i=0;i<fields.length;i++) {
		            System.out.print(fields[i]);
		            header = header + fields[i];
		            if ( i < (fields.length -1) ) {
		                System.out.print(" ");
		                header = header + " ";
		            }
		        }
		        count = 0;
		        System.out.println();
		        bw.write(header);
		        bw.write("\n");
			}
			
			finally {
				bw.close();
			}
		}
		catch (IOException e) {
			e.printStackTrace();
		}
}
    
    int count;
    
    /*
     * (non-Javadoc)
     * @see org.dbgrid.aderis.ResultSink#result(java.lang.String[])
     */
    public void result(String[] values) {
    	BufferedWriter bw = null;
    	
    	try {
			boolean append = true;
			bw = new BufferedWriter(new FileWriter(fileName, append));
						
			try {
				
				for(int i=0;i<values.length;i++) {
		            System.out.print(values[i]);
		            bw.write(values[i]);
		            if ( i < (values.length -1) ) {
		                System.out.print(" ");	
		                bw.write(" ");
		            }
		        }
		        System.out.println();
		        bw.write("\n");
		        count++;
			}
			
			finally {
				bw.close();
			}
		}
		catch (IOException e) {
			e.printStackTrace();
		}
    }
    
    /*
     * (non-Javadoc)
     * @see org.dbgrid.aderis.ResultSink#error(java.lang.String)
     */
    public void error(String message) {
        System.out.println(message);
    }
 
    /*
     * (non-Javadoc)
     * @see org.dbgrid.aderis.ResultSink#finished()
     */
    public void finished() {
       System.out.println("Query finished: (#results=" + count + ")");
       if ( startTime > 0 ) {
           System.out.println("in " + (System.currentTimeMillis()-startTime));
       }
    }
    
    /*
     * Allows a delegate plan listener to be set.
     */
    public void setPlanListener(PlanListener listener) {
        this.listener = listener;
    }
    
    /*
     * (non-Javadoc)
     * @see org.dbgrid.aderis.planner.PlanListener#updatePlan(org.dbgrid.aderis.planner.Plan)
     */
    public void updatePlan(Plan plan) {
       if ( listener != null) {
           listener.updatePlan(plan);
       }
    }
    
   

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

}