package org.gridminer.jspace.queryProcessor;

import java.util.HashSet;
import java.util.Set;

import org.dbgrid.aderis.ConfigurationParameters;
import org.dbgrid.aderis.SPARQLQueryProcessor;
import org.dbgrid.aderis.datasource.DataSource;
import org.gridminer.jspace.manager.PersistentOntology;

import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.rdf.model.ModelMaker;

public class jSpaceQueryProcessor {
	
	//the ontology URI
    public static final String ONT = "http://www.gridminer.org/e-sciencelifecycle/lifecycleontology_v2.0.owl";

    public String DB_URL = "";
    public String DB_USER = "";
    public String DB_PASSWD = "";
    public String DB_TYPE = "";
    public String DB_DRIVER = "";
	
	// First part or the query string 
	static public final String prolog = "PREFIX owl: <http://localhost/LifeCycleOntology.owl#>" ;
	static public final String NL = System.getProperty("line.separator") ;
	/////////////////////////////////////jSpace Configuration/////////////////////////////////////
	
	public String queryString;
	public String queryType;
	
	PersistentOntology po = new PersistentOntology();
	
	public jSpaceQueryProcessor() {
		
	}
	
    /*
     * Constructor
     */
	public jSpaceQueryProcessor(String queryString, String queryType, String db_url, String db_user, String db_paswd, String db_type, String db_driver) {
		this.queryString = queryString;
		this.queryType = queryType;
		
		this.DB_URL = db_url;
		this.DB_USER = db_user;
		this.DB_PASSWD = db_paswd;
		this.DB_TYPE = db_type;
		this.DB_DRIVER = db_driver;
		
		if(queryType.equalsIgnoreCase("SPARQL"))
			this.queryString = buildSparqlQuery(queryString);
		
	}
	
	/*
	 * Builds the SPARQL query String by combining the PREFIX - first part or the query string prolog, 
	 * NL - the line separator, and the query 
	 */
	public String buildSparqlQuery(String query) {
		String queryString = prolog + NL + query;
		return queryString;
	}
	
	/*
	 * returns the SPARQL query String with line numbers
	 */
	public String returnSparqlQuery(String query) {
		return po.serialzeQuery(query);
	}
	
	
	/*
	 * This method queries the local RDF Store defined in the jSpace Configuration.
	 * The Sparql query queryString is submitted to the store.
	 */
	public String queryLocalRDFStore(String formatType) {
		PersistentOntology po = new PersistentOntology();
	    try {
	    	Class.forName(DB_DRIVER);
	    }
	    catch (Exception e) {
	    	System.err.println("Failed to load the driver for the database: " + e.getMessage());
	        System.err.println("Have you got the CLASSPATH set correctly?");
	    }
	    ModelMaker maker = po.getRDBMaker(DB_URL, DB_USER, DB_PASSWD, DB_TYPE, false);
	    ResultSet rs = po.executeSparqlQuery(maker, ONT, this.queryString);
	    //Note the string queryResult is empty when format is RDF or JSON
	    String queryResultSet = po.formatResultSet(rs, formatType, "local");
	    return queryResultSet;
	}
	
	
	/*
	 * This method queries multiple global RDF Stores if available defined in the jSpace Configuration.
	 * The Sparql query queryString is submitted to the store
	 */
	public String queryGlobalRDFStore(String formatType, boolean multipleRDFStores, Set<DataSource> sources) {
			String queryResultSet = "";
	        if(multipleRDFStores) {
	        	
	        	ConfigurationParameters parameters = new ConfigurationParameters();
	            SPARQLQueryProcessor processor = new SPARQLQueryProcessor(parameters,sources);
	            String query = this.queryString;
	            try {            	
 	                processor.executeAsynchronousSELECTQuery(query,new BufferedWriterResultSink(formatType));
	            } 
	            catch ( Exception e ) {
	                System.out.println("error: " + e);
	            }
	        }
	        /*
	         * This was to query the old global RDF store without SPARQL ADERIS
	         */
	        /*
	        else {
	        	PersistentOntology po = new PersistentOntology();
	        	try {
		            Class.forName(DB_DRIVER);
		        }
		        catch (Exception e) {
		            System.err.println("Failed to load the driver for the database: " + e.getMessage());
		            System.err.println("Have you got the CLASSPATH set correctly?");
		        }
		        ModelMaker maker = po.getRDBMaker(g_DB_URL, g_DB_USER, g_DB_PASSWD, g_DB_TYPE, false);
		        ResultSet rs = po.executeSparqlQuery(maker, ONT, this.queryString);
		        queryResultSet = po.formatResultSet(rs, formatType, "global");
	        }
	        */
	        return queryResultSet;
	}
	

}